/***************************************************************************
 * Copyright (c) 2008 by Martin Beck                                       *
 * martin.beck@hpi.uni-potsdam.de                                          *
 *                                                                         *
 * Permission is hereby granted, free of charge, to any person             *
 * obtaining a copy of this software and associated documentation          *
 * files (the "Software"), to deal in the Software without                 *
 * restriction, including without limitation the rights to use,            *
 * copy, modify, merge, publish, distribute, sublicense, and/or sell       *
 * copies of the Software, and to permit persons to whom the               *
 * Software is furnished to do so, subject to the following                *
 * conditions:                                                             *
 *                                                                         *
 * The above copyright notice and this permission notice shall be          *
 * included in all copies or substantial portions of the Software.         *
 *                                                                         *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,         *
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES         *
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND                *
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT             *
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,            *
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING            *
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR           *
 * OTHER DEALINGS IN THE SOFTWARE.                                         *
 ***************************************************************************/

/** \file boolean.h
 *  \brief Functions for the Boolean classes.
 */

#ifndef _BOOLEAN_H_
#define _BOOLEAN_H_

#include "image.h"

/** \brief Type redefinition to indicate Boolean object references. */
typedef ObjectRef BooleanRef;

/** \brief Convert a boolean to an object reference.
 *
 * \param boolean TRUE or FALSE.
 * \return  The singleton object in the image corresponding to the
 *          given boolean value.
 */
static inline BooleanRef Boolean_new(bool boolean) {
  return boolean ? specialObjects->true : specialObjects->false;
}

/** \brief Convert a an boolean object reference to a native boolean.
 *
 * \param boolean One of the two singleton objects corresponding to the
 *                boolean values.
 * \return TRUE or FALSE.
 */
static inline bool Boolean_get(BooleanRef boolean) {
  return specialObjects->true == boolean;
}

#endif // _BOOLEAN_H
