/***************************************************************************
 * Copyright (c) 2008 by Martin Beck                                       *
 * martin.beck@hpi.uni-potsdam.de                                          *
 *                                                                         *
 * Permission is hereby granted, free of charge, to any person             *
 * obtaining a copy of this software and associated documentation          *
 * files (the "Software"), to deal in the Software without                 *
 * restriction, including without limitation the rights to use,            *
 * copy, modify, merge, publish, distribute, sublicense, and/or sell       *
 * copies of the Software, and to permit persons to whom the               *
 * Software is furnished to do so, subject to the following                *
 * conditions:                                                             *
 *                                                                         *
 * The above copyright notice and this permission notice shall be          *
 * included in all copies or substantial portions of the Software.         *
 *                                                                         *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,         *
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES         *
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND                *
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT             *
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,            *
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING            *
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR           *
 * OTHER DEALINGS IN THE SOFTWARE.                                         *
 ***************************************************************************/

#include "communication.h"

#include "object.h"
#include "class.h"
#include "boolean.h"

void Usb_primIsConnected(PrimitiveFrame* frame) {
  if (NXTUsb_isConnected()) {
    Primitive_return(frame, specialObjects->true);
  } else {
    Primitive_return(frame, specialObjects->false);
  }
}

void Usb_primRead(PrimitiveFrame* frame) {
  U32 bytesToRead = 0;
  NXTUsb_readSync((U8*)&bytesToRead, 4);
  ObjectRef string = Class_newWithSize(specialObjects->NXTByteArray, bytesToRead);
  ByteObject* stringPtr = ByteObject_get(string);
  NXTUsb_readSync(stringPtr->bytes, bytesToRead);
  Primitive_return(frame, string);
}

void Usb_primSend(PrimitiveFrame* frame) {
  ObjectRef buffer = Primitive_argumentAt(frame, 0);
  NXTUsb_write(ByteObject_get(buffer)->bytes, Object_getSize(buffer));
  // TODO: The following line always blocks, however usually it should be in here!
  //while (!NXTUsb_dataWritten());
  Primitive_return(frame, buffer);
}

void Bluetooth_primIsWaitingForPIN(PrimitiveFrame* frame) {
  Primitive_return(frame, Boolean_new(NXTBluetooth_isWaitingForPIN()));
}

void Bluetooth_primIsConnectionPending(PrimitiveFrame* frame) {
  Primitive_return(frame, Boolean_new(NXTBluetooth_isConnectionPending()));
}

void Bluetooth_primSendPIN(PrimitiveFrame* frame) {
  StringRef pin = Primitive_argumentAt(frame, 0);
  char pinBuffer[17];
  if (Object_getSize(pin) > 16) {
    NXT_error("Too long pin");
  }
  for (int i = 0; i < Object_getSize(pin); i++) {
    pinBuffer[i] = Object_getInstVar(pin, i);
  }
  pinBuffer[Object_getSize(pin)] = 0;
  NXTBluetooth_sendPin(pinBuffer);
  Primitive_return(frame, pin);
}

void Bluetooth_primAcceptConnection(PrimitiveFrame* frame) {
  NXTBluetooth_acceptConnection(Boolean_get(Primitive_argumentAt(frame, 0)));
  Primitive_return(frame, frame->self);
}

void Bluetooth_primConnectionEstablished(PrimitiveFrame* frame) {
  S32 handle = NXTBluetooth_connectionEstablished();
  if (handle == -1) {
    Primitive_return(frame, specialObjects->nil);
  } else {
    Primitive_return(frame, Integer_new(handle));
  }
}

void Bluetooth_primOpenStream(PrimitiveFrame* frame) {
  S32 handle = Integer_fromReference(Primitive_argumentAt(frame, 0));
  NXTBluetooth_openStream(handle);
  Primitive_return(frame, frame->self);
}

void Bluetooth_primCloseStream(PrimitiveFrame* frame) {
  NXTBluetooth_closeStream();
  Primitive_return(frame, frame->self);
}

void Bluetooth_primIsStreamOpen(PrimitiveFrame* frame) {
  Primitive_return(frame, Boolean_new(NXTBluetooth_isStreamOpen()));
}

void Bluetooth_primReadInto(PrimitiveFrame* frame) {
  ObjectRef buffer = Primitive_argumentAt(frame, 0);
  NXTBluetooth_read(ByteObject_get(buffer)->bytes, Object_getSize(buffer));
  Primitive_return(frame, buffer);
}

void Bluetooth_primWrite(PrimitiveFrame* frame) {
  ObjectRef array = Primitive_argumentAt(frame, 0);
  NXTBluetooth_write(ByteObject_get(array)->bytes, Object_getSize(array));
  Primitive_return(frame, array);
}

void Bluetooth_primDataRead(PrimitiveFrame* frame) {
  Primitive_return(frame, Integer_new(NXTBluetooth_dataRead()));
}

void Bluetooth_primIsDataWritten(PrimitiveFrame* frame) {
  Primitive_return(frame, Boolean_new(NXTBluetooth_dataWritten()));
}

void Bluetooth_primRead(PrimitiveFrame* frame) {
  U32 bytesToRead = 0;
  NXTBluetooth_readSync((U8*)&bytesToRead, 4);
  ObjectRef string = Class_newWithSize(specialObjects->NXTByteArray, bytesToRead);
  ByteObject* stringPtr = ByteObject_get(string);
  NXTBluetooth_readSync(stringPtr->bytes, bytesToRead);
  Primitive_return(frame, string);
}

