/***************************************************************************
 * Copyright (c) 2008 by Martin Beck                                       *
 * martin.beck@hpi.uni-potsdam.de                                          *
 *                                                                         *
 * Permission is hereby granted, free of charge, to any person             *
 * obtaining a copy of this software and associated documentation          *
 * files (the "Software"), to deal in the Software without                 *
 * restriction, including without limitation the rights to use,            *
 * copy, modify, merge, publish, distribute, sublicense, and/or sell       *
 * copies of the Software, and to permit persons to whom the               *
 * Software is furnished to do so, subject to the following                *
 * conditions:                                                             *
 *                                                                         *
 * The above copyright notice and this permission notice shall be          *
 * included in all copies or substantial portions of the Software.         *
 *                                                                         *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,         *
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES         *
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND                *
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT             *
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,            *
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING            *
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR           *
 * OTHER DEALINGS IN THE SOFTWARE.                                         *
 ***************************************************************************/

/** \file communication.h
 * \brief Functions and primitives for communicating via USB and Bluetooth.
 * \author Martin Beck
 */

#ifndef _COMMUNICATION_H_
#define _COMMUNICATION_H_

#include "primitives.h"

/** \name Primitives
 * @{
 */

/** \brief Primitive. Checks wheter a USB connection is present. */
void Usb_primIsConnected(PrimitiveFrame* frame);

/** \brief Primitive. Synchroneously reads a packet from USB. */
void Usb_primRead(PrimitiveFrame* frame);

/** \brief Primitive. Sends a packet to the USB. */
void Usb_primSend(PrimitiveFrame* frame);

/** \brief Primitive. Checks, whether a connecting device wants a PIN. */
void Bluetooth_primIsWaitingForPIN(PrimitiveFrame* frame);

/** \brief Primitive. Checks, whether a new connection is pending. */
void Bluetooth_primIsConnectionPending(PrimitiveFrame* frame);

/** \brief Primitive. Sends the given PIN to the connecting device. */
void Bluetooth_primSendPIN(PrimitiveFrame* frame);

/** \brief Primitive. Starts the accepting handshake of a connection. */
void Bluetooth_primAcceptConnection(PrimitiveFrame* frame);

/** \brief Primitive. Indicates an established connection.
 *
 * Also returns a handle, if a connection was recently established. But only
 * once!
 */
void Bluetooth_primConnectionEstablished(PrimitiveFrame* frame);

/** \brief Primitive. Opens a stream for writing and reading asynchroneously.*/
void Bluetooth_primOpenStream(PrimitiveFrame* frame);

/** \brief Primitive. Closes the currently opened stream. */
void Bluetooth_primCloseStream(PrimitiveFrame* frame);

/** \brief Primitive. Indicates, whether the stream opening has finished. */
void Bluetooth_primIsStreamOpen(PrimitiveFrame* frame);

/** \brief Primitive. Starts asynchroneous reading into the provided buffer. */
void Bluetooth_primReadInto(PrimitiveFrame* frame);

/** \brief Primitive. Returns the bytes read. */
void Bluetooth_primDataRead(PrimitiveFrame* frame);

/** \brief Primitive. Send the provided buffer. */
void Bluetooth_primWrite(PrimitiveFrame* frame);

/** \brief Primitive. Indicates when all data has been sent. */
void Bluetooth_primIsDataWritten(PrimitiveFrame* frame);

/** \brief Primitive. Read a whole NXTalk packet. */
void Bluetooth_primRead(PrimitiveFrame* frame);
/** @} */

#endif // _COMMUNICATION_H_
