/***************************************************************************
 * Copyright (c) 2008 by Martin Beck                                       *
 * martin.beck@hpi.uni-potsdam.de                                          *
 *                                                                         *
 * Permission is hereby granted, free of charge, to any person             *
 * obtaining a copy of this software and associated documentation          *
 * files (the "Software"), to deal in the Software without                 *
 * restriction, including without limitation the rights to use,            *
 * copy, modify, merge, publish, distribute, sublicense, and/or sell       *
 * copies of the Software, and to permit persons to whom the               *
 * Software is furnished to do so, subject to the following                *
 * conditions:                                                             *
 *                                                                         *
 * The above copyright notice and this permission notice shall be          *
 * included in all copies or substantial portions of the Software.         *
 *                                                                         *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,         *
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES         *
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND                *
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT             *
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,            *
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING            *
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR           *
 * OTHER DEALINGS IN THE SOFTWARE.                                         *
 ***************************************************************************/

#include "display.h"

#include "image.h"
#include "primitives.h"
#include "integer.h"

void Display_primScrolling(PrimitiveFrame* frame) {
  ObjectRef on = Primitive_argumentAt(frame, 0);
  if (on == specialObjects->true) {
    NXTDisplay_setScrollMode(TRUE);
  } else if (on == specialObjects->false) {
    NXTDisplay_setScrollMode(FALSE);
  } else {
    NXT_error("No boolean");
  }
  Primitive_return(frame, frame->self);
}

void Display_primClear(PrimitiveFrame* frame) {
  NXTDisplay_clear();
  Primitive_return(frame, frame->self);
}

void Display_primCursorToXY(PrimitiveFrame* frame) {
  S16 y = SmallInteger_fromReference(Primitive_argumentAt(frame, 1));
  S16 x = SmallInteger_fromReference(Primitive_argumentAt(frame, 0));
  if (x < 1 || x > 16 || y < 1 || y > 8) {
    NXT_printNumber(x);
    NXT_printString(" ");
    NXT_printNumber(y);
    NXT_error("Out of display");
  }
  NXTDisplay_setCursor(x - 1, y - 1);
  Primitive_return(frame, frame->self);
}
