/***************************************************************************
 * Copyright (c) 2008 by Martin Beck                                       *
 * martin.beck@hpi.uni-potsdam.de                                          *
 *                                                                         *
 * Permission is hereby granted, free of charge, to any person             *
 * obtaining a copy of this software and associated documentation          *
 * files (the "Software"), to deal in the Software without                 *
 * restriction, including without limitation the rights to use,            *
 * copy, modify, merge, publish, distribute, sublicense, and/or sell       *
 * copies of the Software, and to permit persons to whom the               *
 * Software is furnished to do so, subject to the following                *
 * conditions:                                                             *
 *                                                                         *
 * The above copyright notice and this permission notice shall be          *
 * included in all copies or substantial portions of the Software.         *
 *                                                                         *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,         *
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES         *
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND                *
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT             *
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,            *
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING            *
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR           *
 * OTHER DEALINGS IN THE SOFTWARE.                                         *
 ***************************************************************************/

#include "event.h"

static bool Event_testCondition(int value, int testValue, int condition) {
  switch (condition) {
    case GREATER_THAN: {
      return value >= testValue;
    }
    case LESS_THAN: {
      return value <= testValue;
    }
  }
  return FALSE;
}

bool Event_check(Event* event) {
  U8 condition = SmallInteger_fromReference(event->condition);
  switch (SmallInteger_fromReference(event->type)) {
    case TIME: {
      if ((S32)NXT_getSystemTime() >= Integer_fromReference(event->value)) {
        return TRUE;
      }
      break;
    }
    case BUTTON: {
      if (NXTButton_read() == Integer_fromReference(event->value)) {
        return TRUE;
      }
      break;
    }
    case SENSOR_1:
    case SENSOR_2:
    case SENSOR_3:
    case SENSOR_4: {
      U16 sensor = SmallInteger_fromReference(event->type) - SENSOR_1;
      return Event_testCondition(NXTSensor_read(sensor), Integer_fromReference(event->value), condition);
      break;
    }
    case MOTOR_A:
    case MOTOR_B:
    case MOTOR_C: {
      U16 motor = SmallInteger_fromReference(event->type) - MOTOR_A;
      return Event_testCondition((int)NXTMotor_getRotation(motor), Integer_fromReference(event->value), condition);
      break;
    }
  }
  return FALSE;
}
