/***************************************************************************
 * Copyright (c) 2008 by Martin Beck                                       *
 * martin.beck@hpi.uni-potsdam.de                                          *
 *                                                                         *
 * Permission is hereby granted, free of charge, to any person             *
 * obtaining a copy of this software and associated documentation          *
 * files (the "Software"), to deal in the Software without                 *
 * restriction, including without limitation the rights to use,            *
 * copy, modify, merge, publish, distribute, sublicense, and/or sell       *
 * copies of the Software, and to permit persons to whom the               *
 * Software is furnished to do so, subject to the following                *
 * conditions:                                                             *
 *                                                                         *
 * The above copyright notice and this permission notice shall be          *
 * included in all copies or substantial portions of the Software.         *
 *                                                                         *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,         *
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES         *
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND                *
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT             *
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,            *
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING            *
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR           *
 * OTHER DEALINGS IN THE SOFTWARE.                                         *
 ***************************************************************************/

#include "method.h"

#include "image.h"
#include "integer.h"
#include "primitives.h"

bool Method_isMethod(const Method* self) {
  return self->klass == specialObjects->NXTMethod || self->klass == specialObjects->NXTBlock;
}

void Method_primObjectAt(PrimitiveFrame* frame) {
  U16 index = SmallInteger_fromReference(Primitive_argumentAt(frame, 0)) - 1;
  Method* self = Method_get(frame->self);
  if (index == 0) {
    Primitive_return(frame, self->header);
  } else {
    Primitive_return(frame, self->literals[index - 1]);
  }
}

void Method_primObjectAtPut(PrimitiveFrame* frame) {
  U16 index = SmallInteger_fromReference(Primitive_argumentAt(frame, 0)) - 1;
  ObjectRef value = Primitive_argumentAt(frame, 1);
  Method* self = Method_getForWrite(frame->self);
  if (index == 0) {
    self->header = value;
  } else {
    Object_incReferenceCount(value);
    Object_decReferenceCount(self->literals[index - 1]);
    self->literals[index - 1] = value;
  }
}
