/***************************************************************************
 * Copyright (c) 2008 by Martin Beck                                       *
 * martin.beck@hpi.uni-potsdam.de                                          *
 *                                                                         *
 * Permission is hereby granted, free of charge, to any person             *
 * obtaining a copy of this software and associated documentation          *
 * files (the "Software"), to deal in the Software without                 *
 * restriction, including without limitation the rights to use,            *
 * copy, modify, merge, publish, distribute, sublicense, and/or sell       *
 * copies of the Software, and to permit persons to whom the               *
 * Software is furnished to do so, subject to the following                *
 * conditions:                                                             *
 *                                                                         *
 * The above copyright notice and this permission notice shall be          *
 * included in all copies or substantial portions of the Software.         *
 *                                                                         *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,         *
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES         *
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND                *
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT             *
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,            *
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING            *
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR           *
 * OTHER DEALINGS IN THE SOFTWARE.                                         *
 ***************************************************************************/

#include "methodcache.h"

typedef struct {
  ClassRef klass;
  SymbolRef symbol;
  MethodRef method;
} MethodCacheEntry;

// Must be a power of two
#define METHOD_CACHE_SIZE 128

MethodCacheEntry methodCache[METHOD_CACHE_SIZE];

void MethodCache_reset() {
  for (int i = 0; i < METHOD_CACHE_SIZE; i++) {
    methodCache[i].klass = 0;
  }
}

MethodRef MethodCache_lookup(ClassRef klass, SymbolRef symbol) {
  MethodCacheEntry* entry = &methodCache[(klass >> 1 ^ symbol >> 1) & (METHOD_CACHE_SIZE - 1)];
  if (entry->klass == klass && entry->symbol == symbol) {
    return entry->method;
  } else {
    entry->klass = klass;
    entry->symbol = symbol;
    entry->method = Class_lookupMethod(klass, symbol);
    return entry->method;
  }
}
