/***************************************************************************
 * Copyright (c) 2008 by Martin Beck                                       *
 * martin.beck@hpi.uni-potsdam.de                                          *
 *                                                                         *
 * Permission is hereby granted, free of charge, to any person             *
 * obtaining a copy of this software and associated documentation          *
 * files (the "Software"), to deal in the Software without                 *
 * restriction, including without limitation the rights to use,            *
 * copy, modify, merge, publish, distribute, sublicense, and/or sell       *
 * copies of the Software, and to permit persons to whom the               *
 * Software is furnished to do so, subject to the following                *
 * conditions:                                                             *
 *                                                                         *
 * The above copyright notice and this permission notice shall be          *
 * included in all copies or substantial portions of the Software.         *
 *                                                                         *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,         *
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES         *
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND                *
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT             *
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,            *
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING            *
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR           *
 * OTHER DEALINGS IN THE SOFTWARE.                                         *
 ***************************************************************************/

/** \file methodcache.h
 * \author Martin Beck
 * \brief Functions for the method cache.
 *
 * The method cache is responsible for optimizing the process of looking up
 * methods for a given class and message selector. It stores the method
 * reference found using the combined hashes of the selector and the class.
 * When the next request for the same class and selector combination
 * comes in, it does not need to invoke the full method lookup but can
 * return the method directly.
 *
 * However, the stored data can be overwritten by other lookup results and
 * thus is checked for its correctness before it is used.
 */

#include "class.h"

/** \brief Reinitializes the method cache to contain nothing. */
void MethodCache_reset(void);

/** \brief Looks up a method using the cache.
 *
 * \param klass  The class where to start the lookup process.
 * \param symbol Message selector to look up.
 * \return A reference to the compiled method object found. Nil, if nothing
 *         was found.
 */
MethodRef MethodCache_lookup(ClassRef klass, SymbolRef symbol);
