/***************************************************************************
 * Copyright (c) 2008 by Martin Beck                                       *
 * martin.beck@hpi.uni-potsdam.de                                          *
 *                                                                         *
 * Permission is hereby granted, free of charge, to any person             *
 * obtaining a copy of this software and associated documentation          *
 * files (the "Software"), to deal in the Software without                 *
 * restriction, including without limitation the rights to use,            *
 * copy, modify, merge, publish, distribute, sublicense, and/or sell       *
 * copies of the Software, and to permit persons to whom the               *
 * Software is furnished to do so, subject to the following                *
 * conditions:                                                             *
 *                                                                         *
 * The above copyright notice and this permission notice shall be          *
 * included in all copies or substantial portions of the Software.         *
 *                                                                         *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,         *
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES         *
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND                *
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT             *
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,            *
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING            *
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR           *
 * OTHER DEALINGS IN THE SOFTWARE.                                         *
 ***************************************************************************/

#include "motor.h"

#include "primitives.h"
#include "integer.h"
#include "boolean.h"

void Motor_primTachoCountFor(PrimitiveFrame* frame) {
  U16 motorNumber = SmallInteger_fromReference(Primitive_argumentAt(frame, 0));
  S32 tachoCount = NXTMotor_getRotation(motorNumber);
  if (tachoCount < 0) tachoCount = -tachoCount;
  Primitive_return(frame, Integer_new(tachoCount));
}

void Motor_primResetTachoCountFor(PrimitiveFrame* frame) {
  U16 motorNumber = SmallInteger_fromReference(Primitive_argumentAt(frame, 0));
  NXTMotor_resetRotation(motorNumber);
  Primitive_return(frame, frame->self);
}

void Motor_primRotateAt(PrimitiveFrame* frame) {
  U16 motorNumber = SmallInteger_fromReference(Primitive_argumentAt(frame, 0));
  S8 speed = SmallInteger_fromReference(Primitive_argumentAt(frame, 1));
  NXTMotor_rotate(motorNumber, speed);
  Primitive_return(frame, frame->self);
}

void Motor_primRotateAtByBrakingHard(PrimitiveFrame* frame) {
  U16 motorNumber = SmallInteger_fromReference(Primitive_argumentAt(frame, 0));
  S8 speed = SmallInteger_fromReference(Primitive_argumentAt(frame, 1));
  U32 degrees = Integer_fromReference(Primitive_argumentAt(frame, 2));
  bool hardBrake = Boolean_get(Primitive_argumentAt(frame, 3));
  NXTMotor_rotateByAngle(motorNumber, speed, degrees, hardBrake);
  Primitive_return(frame, frame->self);
}

void Motor_primRotateAtForBrakingHard(PrimitiveFrame* frame) {
  U16 motorNumber = SmallInteger_fromReference(Primitive_argumentAt(frame, 0));
  S8 speed = SmallInteger_fromReference(Primitive_argumentAt(frame, 1));
  U32 milliseconds = Integer_fromReference(Primitive_argumentAt(frame, 2));
  bool hardBrake = Boolean_get(Primitive_argumentAt(frame, 3));
  NXTMotor_rotateForMilliseconds(motorNumber, speed, milliseconds, hardBrake);
  Primitive_return(frame, frame->self);
}

void Motor_primStopBrakingHard(PrimitiveFrame* frame) {
  U16 motorNumber = SmallInteger_fromReference(Primitive_argumentAt(frame, 0));
  bool hardBrake = Boolean_get(Primitive_argumentAt(frame, 1));
  NXTMotor_stop(motorNumber, hardBrake);
  Primitive_return(frame, frame->self);
}
