/***************************************************************************
 * Copyright (c) 2008 by Martin Beck                                       *
 * martin.beck@hpi.uni-potsdam.de                                          *
 *                                                                         *
 * Permission is hereby granted, free of charge, to any person             *
 * obtaining a copy of this software and associated documentation          *
 * files (the "Software"), to deal in the Software without                 *
 * restriction, including without limitation the rights to use,            *
 * copy, modify, merge, publish, distribute, sublicense, and/or sell       *
 * copies of the Software, and to permit persons to whom the               *
 * Software is furnished to do so, subject to the following                *
 * conditions:                                                             *
 *                                                                         *
 * The above copyright notice and this permission notice shall be          *
 * included in all copies or substantial portions of the Software.         *
 *                                                                         *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,         *
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES         *
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND                *
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT             *
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,            *
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING            *
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR           *
 * OTHER DEALINGS IN THE SOFTWARE.                                         *
 ***************************************************************************/

/** \file motor.h
 * \brief Primitives for the Motor class.
 * \author Martin Beck
 */

#ifndef _MOTOR_H_
#define _MOTOR_H_

#include "primitives.h"

/** \name Primitives
 * @{
 */

/** \brief Primitive. Read integrated rotation sensor. */
void Motor_primTachoCountFor(PrimitiveFrame* frame);

/** \brief Primitive. Reset integrated rotation sensor. */
void Motor_primResetTachoCountFor(PrimitiveFrame* frame);

/** \brief Primitive. Rotate the motor with the given power. */
void Motor_primRotateAt(PrimitiveFrame* frame);

/** \brief Primitive. Rotate the motor by the degrees possibly braking hard. */
void Motor_primRotateAtByBrakingHard(PrimitiveFrame* frame);

/** \brief Primitive. Rotate the motor for the time possibly braking hard. */
void Motor_primRotateAtForBrakingHard(PrimitiveFrame* frame);

/** \brief Primitive. Stop the motor possibly braking hard. */
void Motor_primStopBrakingHard(PrimitiveFrame* frame);
/** @} */

#endif // _MOTOR_H_
