/***************************************************************************
 * Copyright (c) 2008 by Martin Beck                                       *
 * martin.beck@hpi.uni-potsdam.de                                          *
 *                                                                         *
 * Permission is hereby granted, free of charge, to any person             *
 * obtaining a copy of this software and associated documentation          *
 * files (the "Software"), to deal in the Software without                 *
 * restriction, including without limitation the rights to use,            *
 * copy, modify, merge, publish, distribute, sublicense, and/or sell       *
 * copies of the Software, and to permit persons to whom the               *
 * Software is furnished to do so, subject to the following                *
 * conditions:                                                             *
 *                                                                         *
 * The above copyright notice and this permission notice shall be          *
 * included in all copies or substantial portions of the Software.         *
 *                                                                         *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,         *
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES         *
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND                *
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT             *
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,            *
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING            *
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR           *
 * OTHER DEALINGS IN THE SOFTWARE.                                         *
 ***************************************************************************/

#include "nxt_api.h"

void NXT_init(void) {
  NXTDisplay_init();
  NXTDisplay_setScrollMode(TRUE);
  NXTMemory_init();
  NXTFlash_init();
  NXTUsb_init();
  NXTBluetooth_init();
}

void NXT_shutdown() {
  NXT_printString("That was all folks");
  NXTMemory_shutdown();
  NXTFlash_shutdown();
  NXTBluetooth_shutdown();
  NXTUsb_shutdown();
  NXTDisplay_shutdown();
  NXTSystem_shutdown();
}

void NXT_error(const char* error_message) {
  while (NXTButton_read() != BUTTON_NONE);
  NXT_printString(error_message);
  while (NXTButton_read() != BUTTON_OK);
  NXT_shutdown();
}

unsigned int seed = 0;

int NXT_rand() {
  long k;
  long s = (long)seed;
  if (s == 0)
    s = 0x12345987;
  k = s / 127773;
  s = 16807 * (s - k * 127773) - 2836 * k;
  if (s < 0)
    s += 2147483647;
  seed = (unsigned int)s;
  return (int)s;
}

void NXTUsb_readSync(U8* buffer, U32 size) {
  U8* bufStart = buffer;
  U8* bufEnd = buffer + size;
  while (bufStart < bufEnd) {
    if (size > NXTUsb_packetSize()) {
      NXTUsb_read(bufStart, NXTUsb_packetSize());
    } else {
      NXTUsb_read(bufStart, size);
    }
    while (!NXTUsb_dataRead());
    bufStart += NXTUsb_dataRead();
    size -= NXTUsb_dataRead();
  }
}

void NXTBluetooth_readSync(U8* buffer, U32 size) {
  NXTBluetooth_read(buffer, size);
  while (NXTBluetooth_dataRead() < size);
}

