/***************************************************************************
 * Copyright (c) 2008 by Martin Beck                                       *
 * martin.beck@hpi.uni-potsdam.de                                          *
 *                                                                         *
 * Permission is hereby granted, free of charge, to any person             *
 * obtaining a copy of this software and associated documentation          *
 * files (the "Software"), to deal in the Software without                 *
 * restriction, including without limitation the rights to use,            *
 * copy, modify, merge, publish, distribute, sublicense, and/or sell       *
 * copies of the Software, and to permit persons to whom the               *
 * Software is furnished to do so, subject to the following                *
 * conditions:                                                             *
 *                                                                         *
 * The above copyright notice and this permission notice shall be          *
 * included in all copies or substantial portions of the Software.         *
 *                                                                         *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,         *
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES         *
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND                *
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT             *
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,            *
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING            *
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR           *
 * OTHER DEALINGS IN THE SOFTWARE.                                         *
 ***************************************************************************/

#include <base/util.h>
#include <base/at91sam7s256.h>
#include <base/core.h>
#include <base/display.h>
#include <base/memmap.h>

#include <base/drivers/avr.h>
#include <base/drivers/systick.h>
#include <base/drivers/motors.h>
#include <base/drivers/sensors.h>
#include <base/drivers/radar.h>
#include <base/drivers/usb.h>
#include <base/drivers/_efc.h>
#include <base/drivers/bt.h>
#include <base/drivers/sound.h>

#define NXT_NO_TYPE_DEFINTIONS

#include "nxt_api.h"

#include "garbagecollection.h"

void NXTSystem_shutdown() {
  /*while (NXTButton_read() != BUTTON_NONE);
  GarbageCollection_printStatistics();
  while (NXTButton_read() != BUTTON_OK);*/
  nx_core_halt();
}

void NXT_printChar(char character) {
  char string[2] = " ";
  string[0]= character;
  NXT_printString(string);
}

U32 NXT_getSystemTime(void) {
  return nx_systick_get_ms();
}

U8 NXTButton_read() {
  return nx_avr_get_button();
}

U16 NXTSensor_read(U8 sensor) {
  return nx_sensors_analog_get(sensor);
}

void NXTSensor_enable(U8 sensor) {
  nx_sensors_analog_enable(sensor);
}

void NXTSensor_disable(U8 sensor) {
  nx_sensors_analog_disable(sensor);
}

void NXTRadar_enable(U8 sensor) {
  nx_radar_init(sensor);
}

void NXTRadar_disable(U8 sensor) {
  nx_radar_close(sensor);
}

U32 NXTRadar_readDistance(U8 sensor, U8 object) {
  return nx_radar_read_distance(sensor, object);
}

bool NXTRadar_detect(U8 sensor) {
  return nx_radar_detect(sensor);
}

S32 NXTMotor_getRotation(U8 motor) {
  return (S32)nx_motors_get_tach_count(motor);
}

void NXTMotor_resetRotation(U8 motor) {
  nx_motors_reset_tach_count(motor);
}

void NXTMotor_rotate(U8 motor, S8 percent) {
  nx_motors_rotate(motor, percent);
}

void NXTMotor_rotateByAngle(U8 motor, S8 percent, U32 angle, bool hardBrake) {
  nx_motors_rotate_angle(motor, percent, angle, hardBrake);
}

void NXTMotor_rotateForMilliseconds(U8 motor, S8 percent, U32 ms, bool hardBrake) {
  nx_motors_rotate_time(motor, percent, ms, hardBrake);
}

void NXTMotor_stop(U8 motor, bool hardBrake) {
  nx_motors_stop(motor, hardBrake);
}

void NXTSystem_installWatchDog(NXTClosure watchdog) {
  nx_systick_install_scheduler(watchdog);
}

void NXTBluetooth_init() {
  nx_bt_init();
  nx_bt_set_friendly_name("NXTalk");
  nx_bt_set_discoverable(TRUE);
}

void NXTBluetooth_shutdown() {
}

void NXTBluetooth_setDiscoverable(bool on) {
  nx_bt_set_discoverable(on);
}

bool NXTBluetooth_isConnectionPending() {
  return nx_bt_connection_pending();
}

bool NXTBluetooth_isWaitingForPIN() {
  return nx_bt_has_dev_waiting_for_pin();
}

void NXTBluetooth_sendPin(char* pin) {
  nx_bt_send_pin(pin);
}

void NXTBluetooth_acceptConnection(bool accept) {
  nx_bt_accept_connection(accept);
}

S32 NXTBluetooth_connectionEstablished() {
  return nx_bt_connection_established();
}

void NXTBluetooth_openStream(S32 handle) {
  nx_bt_stream_open(handle);
}

void NXTBluetooth_closeStream() {
  nx_bt_stream_close();
}

bool NXTBluetooth_isStreamOpen() {
  return nx_bt_stream_opened();
}

void NXTBluetooth_read(U8* data, U32 length) {
  nx_bt_stream_read(data, length);
}

U32 NXTBluetooth_dataRead() {
  return nx_bt_stream_data_read();
}

void NXTBluetooth_write(U8* data, U32 length) {
  nx_bt_stream_write(data, length);
}

bool NXTBluetooth_dataWritten() {
  return nx_bt_stream_data_written();
}

void NXTSpeaker_tone(U32 frequency, U32 ms) {
  nx_sound_freq(frequency, ms);
}

void NXTSpeaker_asyncTone(U32 frequency, U32 ms) {
  nx_sound_freq_async(frequency, ms);
}

// Usb functions
U32 NXTUsb_packetSize() {
  return NX_USB_PACKET_SIZE;
}

/**************************** Display functions ******************************/

void NXTDisplay_init() {
}

void NXTDisplay_shutdown() {
}

void NXTDisplay_clear() {
  nx_display_clear();
}

void NXT_printString(const char* string) {
  nx_display_string(string);
}

void NXT_printNumber(U32 number) {
  nx_display_uint(number);
}

void NXT_printHexNumber(U32 number) {
  nx_display_hex(number);
}

void NXTDisplay_setScrollMode(bool on) {
  nx_display_scroll_ok(on);
}

U8 NXTDisplay_getCursorX(void) {
  return nx_display_cursor_get_pos_x();
}

U8 NXTDisplay_getCursorY(void) {
  return nx_display_cursor_get_pos_y();
}

void NXTDisplay_setCursor(U8 x, U8 y) {
  nx_display_cursor_set_pos(x, y);
}

/************************* End of display functions **************************/

/**************************** USB functions **********************************/
void NXTUsb_init() {
}

void NXTUsb_shutdown() {
}

bool NXTUsb_isConnected() {
  return nx_usb_is_connected();
}

void NXTUsb_read(U8* data, U32 length) {
  nx_usb_read(data, length);
}

U32 NXTUsb_dataRead() {
  return nx_usb_data_read();
}

void NXTUsb_write(U8* data, U32 length) {
  nx_usb_write(data, length);
}

bool NXTUsb_dataWritten() {
  return nx_usb_data_written();
}
/************************ End of USB functions *******************************/

/**************************** Memory functions *******************************/
void NXTMemory_init() {
}

void NXTMemory_shutdown() {
}

U8* NXTMemory_getStart() {
  return NX_USERSPACE_START;
}

U8* NXTMemory_getEnd() {
  return (NX_USERSPACE_START + NX_USERSPACE_SIZE);
}

U32 NXTMemory_getSize() {
  return NX_USERSPACE_SIZE;
}

bool NXTMemory_isRAMAddress(void* address) {
  return address >= (void*)NXTMemory_getStart() &&
      address < (void*)NXTMemory_getEnd();
}

void NXTMemory_copy(void* dest, void* source, U32 length) {
  memcpy(dest, source, length);
}

void NXTMemory_move(void* dest, void* source, U32 length) {
  memmove(dest, source, length);
}
/*********************** End of memory functions *****************************/

/****************************** Flash functions ******************************/
void NXTFlash_init() {
   nx__efc_init();
}

void NXTFlash_shutdown() {
}

void NXTFlash_readPage(U32 page, U8* data) {
  nx__efc_read_page(page, (U32*)data);
}

void NXTFlash_writePage(U8* data, U32 page) {
  nx__efc_write_page((U32*)data, page);
}

U32 NXTFlash_firstPage() {
  return (NXTFlash_getStart() - (U8*)FLASH_BASE_PTR) / EFC_PAGE_BYTES;
}

U32 NXTFlash_lastPage() {
  return EFC_PAGES - 1;
}

U32 NXTFlash_getSize() {
  return (U32)NX_ROM_USERSPACE_END - (U32)FLASH_BASE_PTR;
}

U8* NXTFlash_getStart() {
  return (U8*)((U32)(NX_ROM_USERSPACE_START + EFC_PAGE_BYTES - 1) /
      EFC_PAGE_BYTES * EFC_PAGE_BYTES);
}

U8* NXTFlash_getEnd() {
  return NX_ROM_USERSPACE_END;
}

U32 NXTFlash_getPageSize(void) {
  return EFC_PAGE_BYTES;
}
/************************ End of Flash functions *****************************/
