/***************************************************************************
 * Copyright (c) 2008 by Martin Beck                                       *
 * martin.beck@hpi.uni-potsdam.de                                          *
 *                                                                         *
 * Permission is hereby granted, free of charge, to any person             *
 * obtaining a copy of this software and associated documentation          *
 * files (the "Software"), to deal in the Software without                 *
 * restriction, including without limitation the rights to use,            *
 * copy, modify, merge, publish, distribute, sublicense, and/or sell       *
 * copies of the Software, and to permit persons to whom the               *
 * Software is furnished to do so, subject to the following                *
 * conditions:                                                             *
 *                                                                         *
 * The above copyright notice and this permission notice shall be          *
 * included in all copies or substantial portions of the Software.         *
 *                                                                         *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,         *
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES         *
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND                *
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT             *
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,            *
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING            *
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR           *
 * OTHER DEALINGS IN THE SOFTWARE.                                         *
 ***************************************************************************/

#include "objecttable.h"

#include "object.h"
#include "class.h"
#include "garbagecollection.h"

U32* objectTable;
U32* objectTableStart;
U32* objectTableROMStart;
ObjectRef minReference;
U16 objectCount;

#ifdef LINUX
void* linuxPointers[32768];
#endif

U16 ObjectTable_findFreeIndex() {
  // try to find one in the existing table
  for (int objectReference = MAX_REFERENCE; objectReference >= minReference; objectReference -= 2) {
    if (objectTable[objectReference >> 1] == 0) {
      return objectReference;
    }
  }
  // Add a new object table entry
  if (endOfRAMObjects >= (U8*)(objectTableStart - 1) || endOfROMObjects >= (U8*)(objectTableROMStart - 1)) {
    return 0;
  }
  minReference -= 2;
  objectTableStart--;
  objectTableROMStart--;
  objectTable[minReference >> 1] = 0;
  return minReference;
}

void ObjectTable_encodeClassAndSizeInfo(U16 reference) {
  ObjectRef class = Object_getClass(reference);
  Class* classPtr = Class_get(class);
  U8 sizeBits = 0;
  if (Class_isVariable(classPtr)) {
    sizeBits = Class_getElementSize(classPtr);
  } else {
    sizeBits = Class_getInstVarCount(classPtr);
  }
  U32 encoded = (((sizeBits & 0x1F) << 16) + class) & 0x3FFFFF;
  objectTable[reference >> 1] = (objectTable[reference >> 1] & 0xFFFC00000) | encoded;
}

U16 ObjectTable_decodeClassInfo(U16 reference) {
  return ObjectTable_get(reference) & 0xFFFF;
}

U16 ObjectTable_decodeSizeForNonIndexableObject(U16 reference) {
  return (ObjectTable_get(reference) >> 16) & 0x1F;
}

U16 ObjectTable_decodeElementSize(U16 reference) {
  return ObjectTable_decodeSizeForNonIndexableObject(reference);
}
