/***************************************************************************
 * Copyright (c) 2008 by Martin Beck                                       *
 * martin.beck@hpi.uni-potsdam.de                                          *
 *                                                                         *
 * Permission is hereby granted, free of charge, to any person             *
 * obtaining a copy of this software and associated documentation          *
 * files (the "Software"), to deal in the Software without                 *
 * restriction, including without limitation the rights to use,            *
 * copy, modify, merge, publish, distribute, sublicense, and/or sell       *
 * copies of the Software, and to permit persons to whom the               *
 * Software is furnished to do so, subject to the following                *
 * conditions:                                                             *
 *                                                                         *
 * The above copyright notice and this permission notice shall be          *
 * included in all copies or substantial portions of the Software.         *
 *                                                                         *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,         *
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES         *
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND                *
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT             *
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,            *
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING            *
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR           *
 * OTHER DEALINGS IN THE SOFTWARE.                                         *
 ***************************************************************************/

#include "primitives.h"

#include "interpreter.h"
#include "motor.h"
#include "sensors.h"
#include "system.h"
#include "communication.h"
#include "display.h"
#include "scheduler.h"
#include "garbagecollection.h"

Primitive primitives[PRIMITIVE_COUNT] = {
  &String_primPrint,                            //  0: NXTString class>>#print:
  &Motor_primRotateAt,                          //  1: NXTMotor class>>#motor:speed:
  &BlockFrame_primValue,                        //  2: NXTBlockFrame>>#value
  &SmallInteger_primAdd,                        //  3: NXTSmallInteger>>#+
  &SmallInteger_primPrint,                      //  4: NXTSmallInteger>>#print
  &BlockFrame_primRestart,                      //  5: NXTBlockFrame>>#restart
  &Button_primRead,                             //  6: NXTButton class>>#read
  &Object_primEquals,                           //  7: NXTObject>>#==
  &SmallInteger_primBitAnd,                     //  8: NXTSmallInteger>>#bitAnd:
  &Motor_primRotateAtByBrakingHard,             //  9: NXTMotor class>>#rotate:at:by:brakingHard:
  &Timer_primNow,                               // 10: NXTTimer class>>#now
  &SmallInteger_primLessThan,                   // 11: NXTSmallInteger>>#<
  &Object_primClass,                            // 12: NXTObject>>#class
  &LargeInteger_primPrint,                      // 13: NXTLargePositiveInteger>>#print
  &LargeInteger_primAdd,                        // 14: NXTLargePositiveInteger>>#add
  &Class_primBasicNew,                          // 15: NXTClass>>#basicNew
  &Class_primBasicNewSize,                      // 16: NXTClass>>#basicNew:
  &Object_primSize,                             // 17: NXTObject>>#size
  &SmallInteger_primDivFloor,                   // 18: NXTSmallInteger>>#//
  &SmallInteger_primSub,                        // 19: NXTSmallInteger>>#-
  &Object_primAt,                               // 20: NXTObject>>#at:
  &Object_primAtPut,                            // 21: NXTObject>>#at:put:
  &SmallInteger_primMul,                        // 22: NXTSmallInteger>>#*
  &SmallInteger_primBitXor,                     // 23: NXTSmallInteger>>#bitXor:
  &SmallInteger_primAtRandom,                   // 24: NXTSmallInteger>>#atRandom
  &Object_primPerform,                          // 25: NXTObject>>#perform:, #perform:with:, etc.
  &Usb_primIsConnected,                         // 26: NXTUsb class>>#isConnected
  &Usb_primRead,                                // 27: NXTUsb class>>#read
  &Class_primFirstInstance,                     // 28: NXTClass>>#firstInstance
  &Object_primNextInstance,                     // 29: NXTObject>>#nextInstance
  &Sensor_primEnable,                           // 30: NXTSensor class>>#enable:
  &SmallInteger_primBitShift,                   // 31: NXTSmallInteger>>#bitShift:
  &Display_primScrolling,                       // 32: NXTDisplay>>#scrolling:
  &Usb_primSend,                                // 33: NXTUsb class>>#send:
  &Object_primHalt,                             // 34: NXTObject>>#halt
  &Display_primClear,                           // 35: NXTDisplay>>#clear
  &Object_primHash,                             // 36: NXTObject>>#hash
  &SmallInteger_primMod,                        // 37: NXTSmallInteger>>#{backslash backslash}
  &Object_primPerformWithArguments,             // 38: NXTObject>>#perform:withArguments:
  &Time_primMillisecondClockValue,              // 39: NXTTime class>>#millisecondClockValue
  &LargeInteger_primSub,                        // 40: NXTLargePositiveInteger>>#-
  &LargeInteger_primLessThan,                   // 41: NXTLargePositiveInteger>>#<
  &SmallInteger_primBitOr,                      // 42: NXTSmallInteger>>#bitOr:
  &LargeInteger_primMul,                        // 43: NXTLargePositiveInteger>>#*
  &LargeInteger_primEquals,                     // 44: NXTLargePositiveInteger>>#=
  &Display_primCursorToXY,                      // 45: NXTDisplay>>#cursorToX:Y:
  &Timer_primSleepFor,                          // 46: NXTTimer class>>#sleepFor:
  &Scheduler_primScheduleTo,                    // 47: NXTProcessorScheduler>>#scheduleTo:
  &Scheduler_primSchedulingEnabled,             // 48: NXTProcessorScheduler>>#schedulingEnabled:
  &Method_primObjectAt,                         // 49: NXTMethod>>#objectAt:
  &Method_primObjectAtPut,                      // 50: NXTMethod>>#objectAt:put:
  &SystemDictionary_shutdown,                   // 51: NXTSystemDictionary>>#shutdown
  &SystemDictionary_garbageCollect,             // 52: NXTSystemDictionary>>#garbageCollect
  &Sensor_primRead,                             // 53: NXTSensor class>>#read:
  &Motor_primStopBrakingHard,                   // 54: NXTMotor class>>#stop:brakingHard:
  &Motor_primRotateAtForBrakingHard,            // 55: NXTMotor class>>#rotate:at:for:brakingHard:
  &Motor_primTachoCountFor,                     // 56: NXTMotor class>>#tachoCountFor:
  &Radar_primEnable,                            // 57: NXTRadar class>>#enable:
  &Radar_primDisable,                           // 58: NXTRadar class>>#disable:
  &Radar_primDistanceOfObject,                  // 59: NXTRadar class>>#distance:ofObject:
  &Radar_primIsRadar,                           // 60: NXTRadar class>>#isRadar
  &Sensor_primDisable,                          // 61: NXTSensor class>>#disable:
  &String_primHash,                             // 62: NXTString>>#hash
  &Object_primSetClass,                         // 63: NXTObject>>#class:
  &SmallInteger_primLessThanOrEqual,            // 64: NXTSmallInteger>>#<=
  &Scheduler_primYield,                         // 65: NXTProcessorScheduler>>yield
  &Scheduler_primCheckWaitingProcesses,         // 66: NXTProcessorScheduler>>checkWaitingProcesses
  &Process_primReturnFrom,                      // 67: NXTProcess>>return:from:
  &Motor_primResetTachoCountFor,                // 68: NXTMotor class>>resetTachoCountFor:
  &Bluetooth_primIsWaitingForPIN,               // 69: NXTBluetooth class>>isWaitingForPIN
  &Bluetooth_primIsConnectionPending,           // 70: NXTBluetooth class>>isConnectionPending
  &Bluetooth_primSendPIN,                       // 71: NXTBluetooth class>>sendPIN:
  &Bluetooth_primAcceptConnection,              // 72: NXTBluetooth class>>acceptConnection:
  &Bluetooth_primConnectionEstablished,         // 73: NXTBluetooth class>>connectionEstablished
  &Bluetooth_primOpenStream,                    // 74: NXTBluetooth class>>openStream:
  &Bluetooth_primIsStreamOpen,                  // 75: NXTBluetooth class>>isStreamOpen
  &Bluetooth_primReadInto,                      // 76: NXTBluetooth class>>readInto:
  &Bluetooth_primDataRead,                      // 77: NXTBluetooth class>>dataRead
  &Bluetooth_primWrite,                         // 78: NXTBluetooth class>>write:
  &Bluetooth_primIsDataWritten,                 // 79: NXTBluetooth class>>isDataWritten
  &Bluetooth_primRead,                          // 80: NXTBluetooth class>>read
  &Speaker_primToneAtFor,                       // 81: NXTSpeaker>>toneAt:for:
  &Speaker_primAsyncToneAtFor                   // 82: NXTSpeaker>>asyncToneAt:for:
};

ObjectRef temporarySet[PRIMITIVE_TEMP_SET_SIZE];
U8 temporarySetSize;

ObjectRef Primitive_argumentAt(PrimitiveFrame* frame, U16 index) {
  return Interpreter_stackAt(Interpreter_stackTopIndex() - frame->argumentCount + index + 1);
}

void Primitive_argumentAtPut(PrimitiveFrame* frame, U16 index, ObjectRef value) {
  Interpreter_stackAtPut(Interpreter_stackTopIndex() - frame->argumentCount + index + 1, value);
}

void Primitive_clearStack(PrimitiveFrame* frame) {
  // Pop self and all arguments from stack
  for (int i = 0; i < frame->argumentCount + 1; i++) {
    ObjectRef self = Interpreter_stackPop();
    Object_decReferenceCount(self);
  }
}

void Primitive_return(PrimitiveFrame* frame, ObjectRef value) {
  Object_incReferenceCount(value);
  Primitive_clearStack(frame);
  Interpreter_stackPush(value);
}

ObjectRef Primitive_createIntermediateInstance(ObjectRef class, U16 sizeRequested) {
  ObjectRef object = Class_newWithSize(class, sizeRequested);
  Primitive_addToTemporarySet(object);
  return object;
}

void Primitive_addToTemporarySet(ObjectRef object) {
  if (temporarySetSize == PRIMITIVE_TEMP_SET_SIZE) {
    NXT_error("Primitive root set too large");
  }
  temporarySet[temporarySetSize++] = object;
}

void Primitive_emptyTemporarySet() {
  for (int i = 0; i < temporarySetSize; i++) {
    if (ObjectTable_getReferenceCount(temporarySet[i]) == 0) {
      Object_forceFree(temporarySet[i]);
    }
  }
  temporarySetSize = 0;
}
