/***************************************************************************
 * Copyright (c) 2008 by Martin Beck                                       *
 * martin.beck@hpi.uni-potsdam.de                                          *
 *                                                                         *
 * Permission is hereby granted, free of charge, to any person             *
 * obtaining a copy of this software and associated documentation          *
 * files (the "Software"), to deal in the Software without                 *
 * restriction, including without limitation the rights to use,            *
 * copy, modify, merge, publish, distribute, sublicense, and/or sell       *
 * copies of the Software, and to permit persons to whom the               *
 * Software is furnished to do so, subject to the following                *
 * conditions:                                                             *
 *                                                                         *
 * The above copyright notice and this permission notice shall be          *
 * included in all copies or substantial portions of the Software.         *
 *                                                                         *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,         *
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES         *
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND                *
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT             *
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,            *
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING            *
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR           *
 * OTHER DEALINGS IN THE SOFTWARE.                                         *
 ***************************************************************************/

#include "process.h"

#include "image.h"
#include "scheduler.h"
#include "primitives.h"

ProcessRef Process_new() {
  ProcessRef process = Class_new(specialObjects->NXTProcess);
  ObjectRef stack = Class_newWithSize(specialObjects->NXTArray, STACK_SIZE);
  Process* self = Process_getForWrite(process);
  Process_setStack(self, stack);
  Process_setStackTopIndex(self, -1);
  return process;
}

bool Process_checkEvents(const Process* self) {
  EventRef eventRef = self->waitsFor;
  while (eventRef != specialObjects->nil) {
    Event* event = Event_get(eventRef);
    if (Event_check(event)) {
      return TRUE;
    }
    eventRef = event->next;
  }
  return FALSE;
}

void Process_primReturnFrom(PrimitiveFrame* frame) {
  if (Scheduler_get(specialObjects->Processor)->activeProcess == frame->self) {
    NXT_error("Not possible");
  }
  ObjectRef value = Primitive_argumentAt(frame, 0);
  Frame* returnFrame = Frame_get(Primitive_argumentAt(frame, 1));
  Process* self = Process_getForWrite(frame->self);
  int newStackTopIndex = Frame_getStackPointer(returnFrame) - Frame_getArgumentCount(returnFrame);
  Process_setStackTopIndex(self, newStackTopIndex + 1);
  Process_setCurrentFrame(self, returnFrame->sender);
  Object_setInstVar(self->stack, newStackTopIndex + 1, value);
  Primitive_return(frame, frame->self);
}
