/***************************************************************************
 * Copyright (c) 2008 by Martin Beck                                       *
 * martin.beck@hpi.uni-potsdam.de                                          *
 *                                                                         *
 * Permission is hereby granted, free of charge, to any person             *
 * obtaining a copy of this software and associated documentation          *
 * files (the "Software"), to deal in the Software without                 *
 * restriction, including without limitation the rights to use,            *
 * copy, modify, merge, publish, distribute, sublicense, and/or sell       *
 * copies of the Software, and to permit persons to whom the               *
 * Software is furnished to do so, subject to the following                *
 * conditions:                                                             *
 *                                                                         *
 * The above copyright notice and this permission notice shall be          *
 * included in all copies or substantial portions of the Software.         *
 *                                                                         *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,         *
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES         *
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND                *
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT             *
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,            *
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING            *
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR           *
 * OTHER DEALINGS IN THE SOFTWARE.                                         *
 ***************************************************************************/

#include "scheduler.h"

#include "interpreter.h"
#include "primitives.h"
#include "garbagecollection.h"

bool schedulingEnabled;
bool schedulerRunning;

static void Scheduler_setSchedulingEnabled(bool enabled) {
  schedulingEnabled = enabled;
}

void Scheduler_checkWaitingProcesses(Scheduler* self) {
  ProcessRef last = specialObjects->nil;
  ProcessRef next = specialObjects->nil;
  ProcessRef processRef = self->waitingProcesses;
  while (processRef != specialObjects->nil) {
    Process* process = Process_get(processRef);
    next = process->nextLink;
    if (Process_checkEvents(process)) {
      if (last == specialObjects->nil) {
        self->waitingProcesses = process->nextLink;
      } else {
        Process_getForWrite(last)->nextLink = process->nextLink;
      }
      process->nextLink = self->firstLink;
      self->firstLink = processRef;
      if (self->lastLink == specialObjects->nil) {
        Object_incReferenceCount(processRef);
        Object_decReferenceCount(self->lastLink);
        self->lastLink = processRef;
      }
    }
    last = processRef;
    processRef = next;
  }
}

void Scheduler_primScheduleTo(PrimitiveFrame* frame) {
  ProcessRef newProcess = Primitive_argumentAt(frame, 0);
  // leave self on process' stack for later reactivation and normal go-on
  // now switch to the new process
  Interpreter_switchToProcess(newProcess);
  Scheduler_setActiveProcess(Scheduler_getForWrite(specialObjects->Processor), newProcess);
  // Reenable scheduling, if it was disabled before
  Scheduler_setSchedulingEnabled(TRUE);
}

void Scheduler_primSchedulingEnabled(PrimitiveFrame* frame) {
  bool enabled = Boolean_get(Primitive_argumentAt(frame, 0));
  Scheduler_setSchedulingEnabled(enabled);
  Primitive_return(frame, frame->self);
}

void Scheduler_primYield(PrimitiveFrame* UNUSED(frame)) {
  Interpreter_switchToProcess(Scheduler_get(specialObjects->Processor)->schedulerProcess);
  schedulerRunning = TRUE;
  Scheduler_setSchedulingEnabled(TRUE);
}

void Scheduler_primCheckWaitingProcesses(PrimitiveFrame* frame) {
  Scheduler_checkWaitingProcesses(Scheduler_getForWrite(frame->self));
  Primitive_return(frame, frame->self);
}
