/***************************************************************************
 * Copyright (c) 2008 by Martin Beck                                       *
 * martin.beck@hpi.uni-potsdam.de                                          *
 *                                                                         *
 * Permission is hereby granted, free of charge, to any person             *
 * obtaining a copy of this software and associated documentation          *
 * files (the "Software"), to deal in the Software without                 *
 * restriction, including without limitation the rights to use,            *
 * copy, modify, merge, publish, distribute, sublicense, and/or sell       *
 * copies of the Software, and to permit persons to whom the               *
 * Software is furnished to do so, subject to the following                *
 * conditions:                                                             *
 *                                                                         *
 * The above copyright notice and this permission notice shall be          *
 * included in all copies or substantial portions of the Software.         *
 *                                                                         *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,         *
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES         *
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND                *
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT             *
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,            *
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING            *
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR           *
 * OTHER DEALINGS IN THE SOFTWARE.                                         *
 ***************************************************************************/

#include "sensors.h"

#include "nxt_api.h"
#include "integer.h"
#include "boolean.h"

void Sensor_primEnable(PrimitiveFrame* frame) {
  S16 sensor = SmallInteger_fromReference(Primitive_argumentAt(frame, 0));
  NXTSensor_enable(sensor - 1);
  Primitive_return(frame, frame->self);
}

void Sensor_primRead(PrimitiveFrame* frame) {
  S16 sensor = SmallInteger_fromReference(Primitive_argumentAt(frame, 0));
  Primitive_return(frame, SmallInteger_toReference(NXTSensor_read(sensor - 1)));
}

void Sensor_primDisable(PrimitiveFrame* frame) {
  U16 sensor = SmallInteger_fromReference(Primitive_argumentAt(frame, 0));
  NXTSensor_disable(sensor - 1);
  Primitive_return(frame, frame->self);
}

void Radar_primEnable(PrimitiveFrame* frame) {
  U16 portNumber = SmallInteger_fromReference(Primitive_argumentAt(frame, 0));
  NXTRadar_enable(portNumber - 1);
  Primitive_return(frame, frame->self);
}

void Radar_primDisable(PrimitiveFrame* frame) {
  U16 portNumber = SmallInteger_fromReference(Primitive_argumentAt(frame, 0));
  NXTRadar_disable(portNumber - 1);
  Primitive_return(frame, frame->self);
}

void Radar_primDistanceOfObject(PrimitiveFrame* frame) {
  U16 portNumber = SmallInteger_fromReference(Primitive_argumentAt(frame, 0));
  U16 objectNumber = SmallInteger_fromReference(Primitive_argumentAt(frame, 1));
  U32 distance = NXTRadar_readDistance(portNumber - 1, objectNumber);
  Primitive_return(frame, Integer_new(distance));
}

void Radar_primIsRadar(PrimitiveFrame* frame) {
  U16 portNumber = SmallInteger_fromReference(Primitive_argumentAt(frame, 0));
  Primitive_return(frame, Boolean_new(NXTRadar_detect(portNumber - 1)));
}
