/***************************************************************************
 * Copyright (c) 2008 by Martin Beck                                       *
 * martin.beck@hpi.uni-potsdam.de                                          *
 *                                                                         *
 * Permission is hereby granted, free of charge, to any person             *
 * obtaining a copy of this software and associated documentation          *
 * files (the "Software"), to deal in the Software without                 *
 * restriction, including without limitation the rights to use,            *
 * copy, modify, merge, publish, distribute, sublicense, and/or sell       *
 * copies of the Software, and to permit persons to whom the               *
 * Software is furnished to do so, subject to the following                *
 * conditions:                                                             *
 *                                                                         *
 * The above copyright notice and this permission notice shall be          *
 * included in all copies or substantial portions of the Software.         *
 *                                                                         *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,         *
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES         *
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND                *
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT             *
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,            *
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING            *
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR           *
 * OTHER DEALINGS IN THE SOFTWARE.                                         *
 ***************************************************************************/

/** \file sensors.h
 * \brief Functions and primitives for the different Sensor classes.
 * \author Martin Beck
 */

#ifndef _SENSORS_H_
#define _SENSORS_H_

#include "primitives.h"


/** \name Primitives
 * @{
 */

/** \brief Primitive. Enable the analog sensor on the given port. */
void Sensor_primEnable(PrimitiveFrame* frame);

/** \brief Primitive. Read the analog value on a given sensor port. */
void Sensor_primRead(PrimitiveFrame* frame);

/** \brief Primitive. Enable an analog sensor on the given port. */
void Sensor_primDisable(PrimitiveFrame* frame);

/** \brief Primitive. Enable the radar sensor for a given port. */
void Radar_primEnable(PrimitiveFrame* frame);

/** \brief Primitive. Disable the radar sensor on a given port. */
void Radar_primDisable(PrimitiveFrame* frame);

/** \brief Primitive. Read one of eight radar distances on a given port. */
void Radar_primDistanceOfObject(PrimitiveFrame* frame);

/** \brief Primitive. Check whether the given port is a radar sensor. */
void Radar_primIsRadar(PrimitiveFrame* frame);
/** @} */

#endif // _RADAR_H_
