/***************************************************************************
 * Copyright (c) 2008 by Martin Beck                                       *
 * martin.beck@hpi.uni-potsdam.de                                          *
 *                                                                         *
 * Permission is hereby granted, free of charge, to any person             *
 * obtaining a copy of this software and associated documentation          *
 * files (the "Software"), to deal in the Software without                 *
 * restriction, including without limitation the rights to use,            *
 * copy, modify, merge, publish, distribute, sublicense, and/or sell       *
 * copies of the Software, and to permit persons to whom the               *
 * Software is furnished to do so, subject to the following                *
 * conditions:                                                             *
 *                                                                         *
 * The above copyright notice and this permission notice shall be          *
 * included in all copies or substantial portions of the Software.         *
 *                                                                         *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,         *
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES         *
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND                *
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT             *
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,            *
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING            *
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR           *
 * OTHER DEALINGS IN THE SOFTWARE.                                         *
 ***************************************************************************/

#include "string.h"

#include "image.h"
#include "primitives.h"

static bool Symbol_isBinary(char character) {
  switch (character) {
    case '&':
    case '+':
    case '-':
    case '*':
    case '/':
    case '\\':
    case '%':
    case '<':
    case '>':
    case ',':
    case '=':
    case '~': {
      return TRUE;
      break;
    }
    default: {
      return FALSE;
      break;
    }
  }
}

U8 Symbol_countArguments(SymbolRef symbol) {
  IndexableObject* symbolPtr = (IndexableObject*)Object_get(symbol);
  U8* byte = (U8*)symbolPtr->instanceVariables;
  if (Symbol_isBinary(*byte)) {
    return 1;
  }
  U16 length = symbolPtr->size;
  U8 colons = 0;
  while (length--) {
    if (*byte++ == ':') {
      colons++;
    }
  }
  return colons;
}

void String_print(StringRef string) {
  U16 length = Object_getSize(string);
  U8* byte = ByteObject_get(string)->bytes;
  while (length--) {
    NXT_printChar(*byte++);
  }
}

// NXTString>>#print:
void String_primPrint(PrimitiveFrame* frame) {
  // string to print
  ObjectRef string = Primitive_argumentAt(frame, 0);;
  String_print(string);
  Primitive_return(frame, frame->self);
}

void String_primHash(PrimitiveFrame* frame) {
  int count = Object_getSize(frame->self);
  U8* str = ByteObject_get(frame->self)->bytes;
  U32 hash = 0;
  for (int i = 0; i < count; i++) {
    hash = *str++ + (hash << 6) + (hash << 16) - hash;
  }
  Primitive_return(frame, Integer_new(hash % (SMALLINTEGER_MAX + 1)));
}
