/***************************************************************************
 * Copyright (c) 2008 by Martin Beck                                       *
 * martin.beck@hpi.uni-potsdam.de                                          *
 *                                                                         *
 * Permission is hereby granted, free of charge, to any person             *
 * obtaining a copy of this software and associated documentation          *
 * files (the "Software"), to deal in the Software without                 *
 * restriction, including without limitation the rights to use,            *
 * copy, modify, merge, publish, distribute, sublicense, and/or sell       *
 * copies of the Software, and to permit persons to whom the               *
 * Software is furnished to do so, subject to the following                *
 * conditions:                                                             *
 *                                                                         *
 * The above copyright notice and this permission notice shall be          *
 * included in all copies or substantial portions of the Software.         *
 *                                                                         *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,         *
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES         *
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND                *
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT             *
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,            *
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING            *
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR           *
 * OTHER DEALINGS IN THE SOFTWARE.                                         *
 ***************************************************************************/

/** \file string.h
 * \brief Functions for the String and Symbol classes.
 * \author Martin Beck
 */

#ifndef _STRING_H_
#define _STRING_H_

#include "object.h"

/** \brief Type redefinition to indicate String objects. */
typedef ObjectRef StringRef;

/** \brief Type redefinition to indicate Symbol objects* */
typedef ObjectRef SymbolRef;

/** \brief Count arguments of a symbol.
 *
 * Interprets the given Symbol as selector and counts the arguments
 * it would take. Binary selectors are also taken into account.
 *
 * \param symbol The symbol, which arguments shall be counted.
 * \return The number of arguments, the symbol expects.
 */
U8 Symbol_countArguments(SymbolRef symbol);

/** \brief Prints the given string object.
 *
 * \param string The string to print.
 */
void String_print(StringRef string);

/** \name Primitives
 * @{
 */

/** \brief Prints the string or symbol specified as self. */
void String_primPrint(PrimitiveFrame* frame);

/** \brief Calculates the hash of the string. */
void String_primHash(PrimitiveFrame* frame);
/** @} */

#endif // _STRING_H_
