/***************************************************************************
 * Copyright (c) 2008 by Martin Beck                                       *
 * martin.beck@hpi.uni-potsdam.de                                          *
 *                                                                         *
 * Permission is hereby granted, free of charge, to any person             *
 * obtaining a copy of this software and associated documentation          *
 * files (the "Software"), to deal in the Software without                 *
 * restriction, including without limitation the rights to use,            *
 * copy, modify, merge, publish, distribute, sublicense, and/or sell       *
 * copies of the Software, and to permit persons to whom the               *
 * Software is furnished to do so, subject to the following                *
 * conditions:                                                             *
 *                                                                         *
 * The above copyright notice and this permission notice shall be          *
 * included in all copies or substantial portions of the Software.         *
 *                                                                         *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,         *
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES         *
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND                *
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT             *
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,            *
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING            *
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR           *
 * OTHER DEALINGS IN THE SOFTWARE.                                         *
 ***************************************************************************/

#include "system.h"

#include "integer.h"
#include "garbagecollection.h"
#include "boolean.h"
#include "interpreter.h"

void Timer_primNow(PrimitiveFrame* frame) {
  U16 time = NXT_getSystemTime() / 1000;
  Primitive_return(frame, SmallInteger_toReference(time));
}

void Time_primMillisecondClockValue(PrimitiveFrame* frame) {
  ObjectRef value = Integer_new(NXT_getSystemTime());
  Primitive_return(frame, value);
}

void Timer_primSleepFor(PrimitiveFrame* frame) {
  S16 ms = SmallInteger_fromReference(Primitive_argumentAt(frame, 0));
  U32 time = NXT_getSystemTime();
  while ((S32)NXT_getSystemTime() - (S32)time < ms);
  Primitive_return(frame, frame->self);
}

void Speaker_primToneAtFor(PrimitiveFrame* frame) {
  U16 frequency = SmallInteger_fromReference(Primitive_argumentAt(frame, 0));
  U16 ms = SmallInteger_fromReference(Primitive_argumentAt(frame, 1));
  NXTSpeaker_tone(frequency, ms);
  Primitive_return(frame, frame->self);
}

void Speaker_primAsyncToneAtFor(PrimitiveFrame* frame) {
  U16 frequency = SmallInteger_fromReference(Primitive_argumentAt(frame, 0));
  U16 ms = SmallInteger_fromReference(Primitive_argumentAt(frame, 1));
  NXTSpeaker_asyncTone(frequency, ms);
  Primitive_return(frame, frame->self);
}

void SystemDictionary_shutdown(PrimitiveFrame* frame) {
  bool save = Boolean_get(Primitive_argumentAt(frame, 0));
  Interpreter_shutdown(save);
}

void SystemDictionary_garbageCollect(PrimitiveFrame* frame) {
  GarbageCollection_collect();
  Primitive_return(frame, frame->self);
}

void Button_primRead(PrimitiveFrame* frame) {
  Primitive_return(frame, SmallInteger_toReference(NXTButton_read()));
}
