/***************************************************************************
 * Copyright (c) 2008 by Martin Beck                                       *
 * martin.beck@hpi.uni-potsdam.de                                          *
 *                                                                         *
 * Permission is hereby granted, free of charge, to any person             *
 * obtaining a copy of this software and associated documentation          *
 * files (the "Software"), to deal in the Software without                 *
 * restriction, including without limitation the rights to use,            *
 * copy, modify, merge, publish, distribute, sublicense, and/or sell       *
 * copies of the Software, and to permit persons to whom the               *
 * Software is furnished to do so, subject to the following                *
 * conditions:                                                             *
 *                                                                         *
 * The above copyright notice and this permission notice shall be          *
 * included in all copies or substantial portions of the Software.         *
 *                                                                         *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,         *
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES         *
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND                *
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT             *
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,            *
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING            *
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR           *
 * OTHER DEALINGS IN THE SOFTWARE.                                         *
 ***************************************************************************/

/** \file system.h
 * \brief Functions and primitives for system behavior.
 * \author Martin Beck
 */

#ifndef _SYSTEM_H_
#define _SYSTEM_H_

#include "primitives.h"

/** \name Primitives
 * @{
 */

/** \brief Primitive. Play a tone synchroneously. */
void Speaker_primToneAtFor(PrimitiveFrame* frame);

/** \brief Primitive. Play a ton asynchroneously. */
void Speaker_primAsyncToneAtFor(PrimitiveFrame* frame);

/** \brief Primitive. Returns the current button state as a number. */
void Button_primRead(PrimitiveFrame* frame);

/** \brief Primitive. Returns the current system time in seconds. */
void Timer_primNow(PrimitiveFrame* frame);

/** \brief Primitive. Returns the current system time in milliseconds. */
void Time_primMillisecondClockValue(PrimitiveFrame* frame);

/** \brief Primitive. Sleeps for the given amount of seconds. */
void Timer_primSleepFor(PrimitiveFrame* frame);

/** \brief Primitive. Shuts the whole system down. */
void SystemDictionary_shutdown(PrimitiveFrame* frame);

/** \brief Primitive. Enforces a full garbage collection. */
void SystemDictionary_garbageCollect(PrimitiveFrame* frame);
/** @} */

#endif // _SYSTEM_H_
